##################################################
# ChangeLocalAdminPassword.ps1
# Used to change the local admin passwords for machine
# acounts in Active Directory.
# 
# Created: 11/2/2006
# Author: Tyson Kopczynski
##################################################
param([string] $OUDN)

##################################################
# Functions
##################################################
#-------------------------------------------------
# Set-ChoiceMessage
#-------------------------------------------------
# Usage:        Used to set yes and no choice options.
# $No:          The no message.
# $Yes:         The yes message.

function Set-ChoiceMessage{
    param ($No, $Yes)        
    
    $N = ([System.Management.Automation.Host.ChoiceDescription]"&No") 
    $N.HelpMessage = $No 

    $Y = ([System.Management.Automation.Host.ChoiceDescription]"&Yes") 
    $Y.HelpMessage = $Yes

    Return ($Y,$N) 
    }

#-------------------------------------------------
# New-PromptYesNo
#-------------------------------------------------
# Usage:        Used to display a choice prompt.
# $Caption:     The prompt caption.  
# $Message:     The prompt message.
# $Choices:     The object catagory.  

function New-PromptYesNo{
    param ($Caption, $Message, 
        [System.Management.Automation.Host.ChoiceDescription[]]$Choices)
     
    $Host.UI.PromptForChoice($Caption, $Message, $Choices, 0) 
    }

##################################################
# Main
##################################################
#--------------------
# Load Libraries
#--------------------
. .\LibraryGen.ps1
. .\LibraryCrypto.ps1

#--------------------
# Set Config Vars
#--------------------
$ScriptName = "ChangeLocalAdminPassword.ps1"
$ScriptUsage = "Used to change the local admin passwords on machines."
$ScriptCommand = "$ScriptName -OUDN value"
$ScriptParams = "OUDN = The distinguished name of the OU where" `
    + " the machines are located."
$ScriptExamples = "$ScriptName ""OU=Servers,DC=companyabc,DC=com"""
$ErrorLogName = "ChangeLocalAdminPassword_Errors.log"
$Date = Date

#--------------------
# Verify Required Parameters
#--------------------
if ($args[0] -match '-(\?|(h|(help)))'){
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

if (!$OUDN){
    write-host
    write-host "Please specify the OU machines are located in!" `
            -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

#--------------------
# Define DataTable
#--------------------
$ServersTable = new-object System.Data.DataTable
$ServersTable.TableName = "Servers"
[Void]$ServersTable.Columns.Add("Name")
[Void]$ServersTable.Columns.Add("Status")

#--------------------
# Begin Script
#--------------------
# Setup ErrorLog
$ScriptName + " Ran on: " + $Date | out-file $ErrorLogName

write-host
Get-ScriptHeader $ScriptName $ScriptUsage
write-host

.{
    trap{write-host `t "[ERROR]" -Foregroundcolor Red; 
        throw write-host $_ -Foregroundcolor Red;
        Break}
  
    write-host "Domain Connection" -NoNewLine
    
    # You need to test for a domain connection
    $Domain = Get-CurrentDomain
    
    # You then return the domain's name
    write-host `t $Domain.Name -Foregroundcolor Green
}
    
write-host "Checking OU Name" -NoNewLine

if (!(Get-ADObject "distinguishedName" $OUDN "organizationalUnit")){
    write-host `t "Is not valid!" -Foregroundcolor Red
    write-host
    Break
    }
else{
    write-host `t "[OK]" -Foregroundcolor Green
    }

#--------------------
# Get Password
#--------------------
$Choices = Set-ChoiceMessage "No" "Yes"
$Prompt = New-PromptYesNo "Question:" `
    "Do you want me to generate a random password?" $Choices

while(!$Password){
    
    trap{write-host "You need to input an integer!" `
        -Foregroundcolor Red; Continue}
    
    if ($Prompt -eq 0){
        write-host
        [int]$Length = read-host "Please enter the password length"
        
        if ($Length -gt 0){
            &{
                $Temp = New-RandomPassword $Length
                
                write-host
                write-host "Your new random password is:" `
                    -Foregroundcolor White
                
                [System.Runtime.InteropServices.Marshal]::PtrToStringAuto( `
                [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR( `
                    $Temp))
                
                $Prompt = New-PromptYesNo "Question:" `
                    "Is this password ok?" $Choices

                if ($Prompt -eq 0){
                    $Script:Password = $Temp
                    }
            }               
            }
        else{
            write-host "Password length needs to be longer then 0!" `
                -Foregroundcolor Red
            }
        }
    else{
        write-host
        $Password = read-host "Then please enter a password" -AsSecureString
        }
    }

#--------------------
# Get computers and status
#--------------------
write-host
write-host "Getting Server Info" -NoNewLine

&{
    trap{write-host `t "[ERROR]" -Foregroundcolor Red; 
        throw write-host $_ -Foregroundcolor Red;
        Break}
    
    $Root =
        new-object DirectoryServices.DirectoryEntry "LDAP://$OUDN"

    $Searcher = new-object DirectoryServices.DirectorySearcher
    $Searcher.SearchRoot = $Root
    $Searcher.PageSize = 1000

    $SearchItem = "CN" 
    $SearchValue = "*" 
    $SearchClass = "Computer" 
    $SearchCat = "*" 

    $Searcher.Filter =
        "(&($($SearchItem)=$($SearchValue))(objectClass=$( `
        $SearchClass))(objectCategory=$($SearchCat)))" 

    $Script:Computers = $Searcher.FindAll()
}

write-host `t "[DONE]" -Foregroundcolor Green

write-host "Getting Status Info" -NoNewLine

$Computers | foreach-object -Begin {$i=0;} `
    -Process {&{$Ping = new-object Net.NetworkInformation.Ping;
            $dNSHostName = $_.GetDirectoryEntry().dNSHostName.ToString();
            trap{"Ping [ERROR]: " + $dNSHostName + " $_" | out-file `
                $ErrorLogName -Append; Continue};
            $Result = $Ping.Send($dNSHostName);
            if ($Result.Status -eq "Success"){  `
                [Void]$ServersTable.Rows.Add($dNSHostName, "Online")} `
            else{[Void]$ServersTable.Rows.Add($dNSHostName, "Offline")};
            $i = $i+1;
            write-progress -Activity "Pinging Servers - $($dNSHostName)" `
                -Status "Progress:" `
                -PercentComplete ($i / $Computers.Count * 100)}}

write-host `t "[DONE]" -Foregroundcolor Green

# Write status info to ErrorLog
$ServersTable | out-file $ErrorLogName -Append

write-host "Changing Passwords"  -NoNewLine

$Password = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto( `
    [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR( `
    $Password))

$OnlineServers = $ServersTable.Select("Status = 'Online'")

foreach ($Server in $OnlineServers) {
    &{
        write-progress -Activity "Getting Users - $($Server.Name)" `
            -Status "Stand by..."

        $Users = get-wmiobject -ErrorVariable Err -ErrorAction `
                    SilentlyContinue Win32_UserAccount -Computer $Server.Name
        
        write-progress -Activity "Getting Users - $($Server.Name)" `
            -Status "Done" -completed $True
        
        if ($Err.Count -ne 0){
            "Getting Users [ERROR]: " + $Server.Name + " " + $Err | out-file `
                $ErrorLogName -Append
            }
        else{
            foreach ($User in $Users){
                if ($User.SID.EndsWith("-500") -eq $True){
                    write-progress -Activity `
                        "Changing Password - `$($User.Name)" `
                        -Status "Stand by..."
                    
                    trap{"Change Password [ERROR]: " + `
                        $Server.Name + " " + $_ | out-file `
                        $ErrorLogName -Append; Continue}

                    $WinNTUser =
                        new-object System.DirectoryServices.DirectoryEntry( `
                        "WinNT://" +  $Server.Name + "/" + $User.Name)
                    
                    $WinNTUser.SetPassword($Password)
                    $Null = $WinNTUser.SetInfo
                    
                    write-progress -Activity `
                        "Changing Password - $($User.Name)" `
                        -Status "Done" -completed $True
                    }
                }
            }
    }
    }

write-host `t "[DONE]" -Foregroundcolor Green
write-host
write-host "Script is now DONE!" -Foregroundcolor Green
write-host "Check the $ErrorLogName for errors." -Foregroundcolor Yellow
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUcVvE2Lxg5aLfh+itB6N+pD5u
# LMqgggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBTIEe7LgpbmeRsVbbr/t4m5VNJLKDANBgkqhkiG9w0BAQEFAASCAQBd
# X/hcICE44UHAMsrNCePHUE7jM52yMLuxLlSDnA97ajl4OmeN15X6NsGOM54XiTfF
# KU8Jsa381RzL2SCu5fPCrPMkDUmpfwJTAPYblY4XSuLkRwgrRezAVvGTfKixPe5p
# 3NboZoEVPgBQAWGYkUScgm1noVyZ7bX1LDFrNw+AuM3sm34AmrMZK1BhPNzHkzvW
# KwROhwOayYyPs0SEfdWKmvPTXjIm1HJ+gORAKmueygXXLccY/4r67ATuSg0iTdxM
# VqHTkD9PKOdYrmublRXPbMGgqq5xD5l3hmECHD13TFq29y3cfGHloZGjFAI5fVI2
# dIbmgS9VMDVN2Qdec7za
# SIG # End signature block
