##################################################
# LibraryGen.ps1
# A library file that contains general script functions. 
#
# Created: 10/8/2006
# Author: Tyson Kopczynski
# Version: 1.0
##################################################
#-------------------------------------------------
# Get-ScriptHeader
#-------------------------------------------------
# Usage:        Generates the script header statement.
# $Name:        The name of the script.
# $Usage:       What the script is used for.   

function Get-ScriptHeader{
    param ($Name, $Usage)

    $Date = Date
    
    $Text = "###################################### `n"
    $Text += "# Script:  $Name `n"
    $Text += "# Usage:   $Usage `n"
    $Text += "# User:    $Env:username `n"
    $Text += "# Date:    $Date `n"
    $Text += "######################################"
    
    write-host $Text
    }
    
#-------------------------------------------------
# Show-ScriptUsage
#-------------------------------------------------
# Usage:        Used to show script usage information.
# $Usage:       Script command usage.
# $Options:     Parameters that are allowed for the script.
# $Examples:    Script execution examples.

function Show-ScriptUsage{
    param ($Usage, $Options, $Examples)

    write-host   
    write-host "Usage:"
    write-host $Usage
    write-host 
    write-host "Options:"
    $Options | foreach{write-host $_}
    write-host
    write-host "Examples:"
    $Examples | foreach{write-host $_}
    }
    
#-------------------------------------------------
# Out-Email
#-------------------------------------------------
# Usage:        Used to show script usage information. 

function Out-Email{
    param ($Server, $From, $To, $Subject, $Body)

    $SMTPClient = new-object System.Net.Mail.SmtpClient
    $SMTPClient.Host = $Server
    $SMTPClient.Send($From, $To, $Subject, $Body)
    }

#-------------------------------------------------
# Get-CurrentDomain
#-------------------------------------------------
# Usage:        Used to get the current domain.

function Get-CurrentDomain{
    [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain()
    }

#-------------------------------------------------
# Get-ADObject
#-------------------------------------------------
# Usage:        Used to retrieve an object from Active Directory.
# $Item:        The object item type. (sAMAccountName or distinguishedName)  
# $Name:        The name of the object. (sAMAccountName or distinguishedName)  
# $Cat:         The object category.   

function Get-ADObject{
    param ($Item, $Name, $Cat)

    trap{Continue}
    
    $Searcher = new-object DirectoryServices.DirectorySearcher

    $SearchItem = "$Item" 
    $SearchValue = "$Name" 
    $SearchCat = "$Cat" 

    $Searcher.Filter =
        "(&($($SearchItem)=$($SearchValue))(objectCategory=$($SearchCat)))"
    
    $Searcher.FindOne().GetDirectoryEntry() 
    }
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUpLeFI+HFcfqXciWCGJungEoo
# p0ygggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBTxZdE7aZFLvbapwuX+CteygOmJOTANBgkqhkiG9w0BAQEFAASCAQA7
# rk+/cExihuJk6wRew5Ncr71A1bQMjc4RbDj/yLm7FTM3TxuOu1pf6KWI/79ZBbxo
# G4djyENF1ftX/bRbHHizFBMX4ccOozjvlLeOPpVWuNti5DunasjzfWeFoz+TrtFN
# ekhAxFv+c4kx4nBtTyTJlHeDsKjsWKwj0ovzPVEkUkRwYiUi7WUQa/xRVW+4l9sE
# X1OaNt/ErtFSnVgkUMM7cC2fqTS1fFSuWmoMGZ9lqp0wcmSd7agbFtgO/4af4Irn
# u0WtsDLii2/xZyWECWeyrLNDPAmP2+y4td1ki98frdn1JnYKXkU0Ux4IppyDKpt9
# A+K7iNk7IDGZQUO6Wkui
# SIG # End signature block
