##################################################
# ProvisionExchangeUsers.ps1
# Used to provision Exchange users based on CSV import file.
# 
# Created: 10/21/2006
# Author: Tyson Kopczynski
##################################################
param([string] $UPNSuffix, [string] $OUDN, [string] $ImportFile)

##################################################
# Main
##################################################
#--------------------
# Load Libraries
#--------------------
. .\LibraryGen.ps1

#--------------------
# Set Config Vars
#--------------------
$ScriptName = "ProvisionExchangeUsers.ps1"
$ScriptUsage = "Used to provision Exchange users based on CSV import file."
$ScriptCommand = "$ScriptName -UPNSuffix value -OUDN value -ImportFile value"
$ScriptParams = "UPNSuffix = The new users UPN suffix.", `
    "OUDN = The distinguishedName of the OU to create users in.", `
    "ImportFile = The import CSV file path/filename."
$ScriptExamples = "$ScriptName ""companyabc.com""" `
    + " ""OU=Accounts,DC=companyabc,DC=com""" `
    + " ""users.csv"""
$ErrorLogName = "ProvisionExchangeUsers.log"
$Date = Date

#--------------------
# Verify Required Parameters
#--------------------
if ($args[0] -match '-(\?|(h|(help)))'){
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

if (!$UPNSuffix){
    write-host
    write-host "Please specify the UPN suffix!" -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

if (!$OUDN){
    write-host
    write-host "Please specify the OU to create users in!" `
            -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

if (!$ImportFile){
    write-host
    write-host "Please specify the import CSV file name!" `
            -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

#--------------------
# Begin Script
#--------------------
# Setup ErrorLog
$ScriptName + " Ran on: " + $Date | out-file $ErrorLogName

write-host
Get-ScriptHeader $ScriptName $ScriptUsage
write-host
write-host "Domain Connection" -NoNewLine

.{
    trap{write-host `t "[ERROR]" -Foregroundcolor Red; 
        throw write-host $_ -Foregroundcolor Red;
        Break}
  
    write-host "Domain Connection" -NoNewLine
    
    # You need to test for a domain connection
    $Domain = Get-CurrentDomain
    
    # You then return the domain's name
    write-host `t $Domain.Name -Foregroundcolor Green
}

write-host "Checking OU Name" -NoNewLine

if (!(Get-ADObject "distinguishedName" $OUDN "organizationalUnit")){
    write-host `t "Is not valid!" -Foregroundcolor Red
    write-host
    Break
    }
else{
    write-host `t "[OK]" -Foregroundcolor Green
    }

write-host "Checking Import File" -NoNewLine

if (!(test-path $ImportFile -pathType Leaf)){
    throw write-host `t "Is not a valid file!" -Foregroundcolor Red
    }
else{
    write-host `t "[OK]" -Foregroundcolor Green
    }

#--------------------
# Get Password
#--------------------
write-host
$Password = read-host "Please enter password" -AsSecureString

#--------------------
# Create mailboxes
#--------------------
write-host
write-progress -Activity "Adding Users" -Status "Stand by..."

$Users = import-csv $ImportFile

$Users | foreach-object -Begin {$i=0;} `
        -Process {$FName = $_.FName;
            $LName = $_.LName;
            $Alias = $_.Alias;
            $Database = $_.Database;
            $UPN = $Alias + "@" + $UPNSuffix;
            $Name = $FName + " " + $LName;
            $Null = new-mailbox -Name $Name -Database $Database `
                -OrganizationalUnit $OUDN -UserPrincipalName $UPN `
                -Password $Password -ResetPasswordOnNextLogon $True `
                -Alias $Alias -DisplayName $Name -FirstName $FName `
                -LastName $LName -ErrorVariable Err -ErrorAction `
                SilentlyContinue;
            if ($Err.Count -ne 0){ `
                "Add User [ERROR]: " + $Alias + " " + $Err | `
                out-file $ErrorLogName -Append};
            $i = $i+1;
            write-progress -Activity "Adding Users" -Status "Progress:" `
                -PercentComplete ($i / $Users.Count * 100)}

write-host "Script is now DONE!" -Foregroundcolor Green
write-host "Check the $ErrorLogName for errors." -Foregroundcolor Yellow 
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU/uVJrH6J8/LHKOMs0pk4aJ6L
# x3qgggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBT6yrOB8O5BZMr2Km0DazHZMec0GTANBgkqhkiG9w0BAQEFAASCAQAG
# VqskEgIDfsOtn1kKsQfpUSuxOQ8udVqtF2tXGlUOCzMs9wgYFY7lGj9duUzVrijN
# 5CZaDRgd0i9zrk8vhTejPnby2d02+Js3E27OcGlGu875p1cKbPUp8bJdkkJQEJsc
# 8Z0pezCtisbKUcBvis5KSMfd6YrgSP2IPa1Nip7zHysx0di2HZzD+FoFAXNsLYTN
# hmsQ8wnno6SsjgKt1E/SXh1TonXrpQ7KWh7neen+PwZX+naFm+xqrXEqNpV5Rtfi
# Kk4KV9+K0zIihMmCERZ+fo7Td9S9SIkgy63MbRz0eG+D5ZtG5zP8MtVF+F8bcCyg
# 130DU7YF5xv3Do+w2d5V
# SIG # End signature block
