##################################################
# IsGroupMember.ps1
# Used to check if users are members of a 
# specified group.
#
# Created: 10/21/2006
# Author: Tyson Kopczynski
##################################################
param([string] $GroupName, [string] $ImportFile, [string] $ExportFile)

##################################################
# Functions
##################################################
#-------------------------------------------------
# Get-ScriptHeader
#-------------------------------------------------
# Usage:        Generates the script header statement.
# $Name:        The name of the script.
# $Usage:       What the script is used for.   

function Get-ScriptHeader{
    param ($Name, $Usage)

    $Date = Date
    
    $Text = "###################################### `n"
    $Text += "# Script       $Name `n"
    $Text += "# Usage:       $Usage `n"
    $Text += "# User:        $Env:username `n"
    $Text += "# Date:        $Date `n"
    $Text += "######################################"
    
    $Text
    }

#-------------------------------------------------
# Show-ScriptUsage
#-------------------------------------------------
# Usage:        Used to show script usage information.

function Show-ScriptUsage{
    write-host   
    write-host "Usage: ISGroupMember -GroupName value" `
                    "-ImportFile value -ExportFile value"
    write-host 
    write-host "Options:"
    write-host
    write-host "GroupName `t: The name of the group to check."
    write-host "ImportFile `t: The import CSV file path/filename."
    write-host "ExportFile `t: [Optional] The export CSV file." `
                    "path/filename."
    write-host
    write-host "CSV Format:"
    write-host "sAMAccountName"
    write-host
    write-host "Example:"
    write-host 'ISGroupMember.ps1 "mygroup"' `
                    '"users.csv" "results.csv"'
    write-host
    }

#-------------------------------------------------
# Get-CurrentDomain
#-------------------------------------------------
# Usage:        Used to get the current domain.

function Get-CurrentDomain{
    [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain()
    }

#-------------------------------------------------
# Get-ADObject
#-------------------------------------------------
# Usage:        Used to retrieve an object from Active Directory.
# $Item:        The object item type. (sAMAccountName or distinguishedName)  
# $Name:        The name of the object. (sAMAccountName or distinguishedName)  
# $Cat:         The object category.   

function Get-ADObject{
    param ($Item, $Name, $Cat)

    trap{Continue}
    
    $Searcher = new-object DirectoryServices.DirectorySearcher

    $SearchItem = "$Item" 
    $SearchValue = "$Name" 
    $SearchCat = "$Cat" 

    $Searcher.Filter = `
        "(&($($SearchItem)=$($SearchValue))(objectCategory=$($SearchCat)))"
    
    $Searcher.FindOne().GetDirectoryEntry() 
    }

##################################################
# Main
##################################################
#--------------------
# Set Config Vars
#--------------------
$ScriptName = "IsGroupMember"
$ScriptUsage = "Used to check if users are members of a specified group."

#--------------------
# Verify Required Parameters
#--------------------
if ($args[0] -match '-(\?|(h|(help)))'){
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage
    Return
    }
    
if (!$GroupName){
    write-host
    write-host "Please specify the group name!" -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage
    Return
    }

if (!$ImportFile){
    write-host
    write-host "Please specify the import CSV file name!" -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage
    Return
    }

#--------------------
# Begin Script
#--------------------
write-host
Get-ScriptHeader $ScriptName $ScriptUsage
write-host

.{
    trap{write-host `t "[ERROR]" -Foregroundcolor Red; 
        throw write-host $_ -Foregroundcolor Red;
        Break}
  
    write-host "Domain Connection" -NoNewLine
    
    # You need to test for a domain connection
    $Domain = Get-CurrentDomain
    
    # You then return the domain's name
    write-host `t $Domain.Name -Foregroundcolor Green
}  

write-host "Checking Group Name" -NoNewLine 

# Now get the group
$Group = Get-ADObject "sAMAccountName" $GroupName "Group"

if (!$Group){
    write-host `t "Is not valid!" -Foregroundcolor Red
    write-host
    Break
    }
else{
    write-host `t "[OK]" -Foregroundcolor Green
    }

write-host "Checking Import File" -NoNewLine
    
if (!(test-path $ImportFile -pathType Leaf)){
    write-host `t "Is not a valid file!" -Foregroundcolor Red
    write-host
    Break
    }
else{
    write-host `t "[OK]" -Foregroundcolor Green
    }

#--------------------
# Check Each User's Group Membership
#--------------------
$Users = import-csv $ImportFile

foreach ($User in $Users){
    &{
        $sAMAccountName = $User.sAMAccountName
        
        $ADUser = Get-ADObject "sAMAccountName" $sAMAccountName "User"

        if ($ADUser){
            [string]$DN = $ADUser.distinguishedName

            $IsMember = $Group.Member | `
                where {$_ -eq $DN}

            if ($IsMember){
                add-member -inputObject $User -membertype noteProperty `
                    -name "IsMember" -value "Yes"
                }
            else{
                add-member -inputObject $User -membertype noteProperty `
                    -name "IsMember" -value "No"
                }
            }
        else{
            # What if the user doesn't exist?
            add-member -inputObject $User -membertype noteProperty `
                -name "IsMember" -value "Doesn't Exist"
            }
    }
    }

if (!$ExportFile){
    $Users
    }
else{
    write-host
    write-host "Exported Data To: " -NoNewLine
    $Users | export-csv $ExportFile
    write-host "$ExportFile" -Foregroundcolor Green 
    write-host
    }
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUy1n7rKhbZqu0H5dewDu77fum
# dTKgggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBTQUQcfLc6X8mdKusE8/xzUZj50ITANBgkqhkiG9w0BAQEFAASCAQBO
# sL1eDzYZM8wIN20YuBKDDWxv1zoQ0d8bXcHlNIJ7rgfpQE18/dPj91d8FJf/Ecx4
# /MBjk5VXBMO68lvG3s6jtvzs7jWid0APuNZHEnC5f6CvRIMQU8MC4xkbTBp+XtgB
# jHECf5rB/ZoIbAQetm6sEZDZVk1lTO8PT0WUMDFfZZFQFDabjn0DVF64pH8mHb2a
# 7BGQ44Efo9jAIza+yxre2k3ELjabdfeS8u5bd40TP/EG2GfqJiXDKVTr1b1I4dM4
# dKah+9rG6Yckx72KD+X7GXogd1ntBOS7IeGiubwtIARZbxZIxvC+W4MO0qqgzhZ2
# h0XDHao2KM+RkVd5hgMk
# SIG # End signature block
